"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cliEffect = void 0;
const effect_1 = require("effect");
const Chunk_1 = require("effect/Chunk");
const Args = __importStar(require("@effect/cli/Args"));
const CliApp = __importStar(require("@effect/cli/CliApp"));
const Command = __importStar(require("@effect/cli/Command"));
const Options = __importStar(require("@effect/cli/Options"));
const Span = __importStar(require("@effect/cli/HelpDoc/Span"));
const HelpDoc = __importStar(require("@effect/cli/HelpDoc"));
const ValidationError = __importStar(require("@effect/cli/ValidationError"));
const core_1 = require("./core");
const packageJson = __importStar(require("../package.json"));
const types_1 = require("./types");
const Function_1 = require("effect/Function");
const GitCreateJiraBranch = effect_1.Data.case();
const mainCommand = (0, effect_1.pipe)(Command.make('git-create-jira-branch', {
    options: Options.all({
        baseBranch: Options.withDescription(Options.optional(Options.alias(Options.text('baseBranch'), 'b')), 'Base branch to create the new branch from'),
        reset: Options.withDescription(Options.alias(Options.boolean('reset'), 'r'), 'Reset the branch if it already exists'),
        version: Options.withDescription(Options.alias(Options.boolean('version'), 'v'), 'Show version information'),
        help: Options.withDescription(Options.alias(Options.boolean('help'), 'h'), 'Show this help text'),
    }),
    args: Args.addDescription(Args.atMost(Args.text({ name: 'jira-key' }), 1), 'The Jira ticket key to create a branch for (e.g. FOOX-1234)'),
}), Command.withHelp(HelpDoc.p(`Fetches the given Jira ticket and creates an aproriately named branch for it.
The branch type (bug or feat) is determined by the ticket type. The branch name
is based on the ticket summary.`)), Command.map((args) => {
    return GitCreateJiraBranch({
        version: args.options.version,
        baseBranch: args.options.baseBranch,
        reset: args.options.reset,
        jiraKey: (0, Chunk_1.get)(args.args, 0),
    });
}));
const cli = CliApp.make({
    name: packageJson.name,
    version: packageJson.version,
    command: mainCommand,
    summary: Span.text(packageJson.description),
});
const cliEffect = (args) => CliApp.run(cli, args, (command) => {
    if (command.version) {
        return effect_1.Console.log(`${cli.name} v${cli.version}`);
    }
    return effect_1.Option.match(command.jiraKey, {
        onSome: (jiraKey) => effect_1.Effect.flatMap((0, core_1.gitCreateJiraBranch)(jiraKey, command.baseBranch, command.reset), (0, Function_1.compose)((0, types_1.matchGitCreateJiraBranchResult)({
            onCreatedBranch: (branch) => `Successfully created branch: '${branch}'`,
            onSwitchedBranch: (branch) => `Switched to already existing branch: '${branch}'`,
            onResetBranch: (branch) => `Reset branch: '${branch}'`,
        }), effect_1.Console.log)),
        onNone: () => printDocs(HelpDoc.p(Span.error('No Jira Key provided'))),
    });
}).pipe(effect_1.Effect.catchIf(ValidationError.isValidationError, (_) => 
// Validation errors are already handled by the CLI
effect_1.Effect.succeed(undefined)), effect_1.Effect.catchAll((e) => printDocs(HelpDoc.p(Span.error(e.message)))));
exports.cliEffect = cliEffect;
const printDocs = (doc) => effect_1.Console.log(HelpDoc.toAnsiText(doc));
